@@ -8,25 +8,24 @@
 #include <memory>
 #include <mutex>
 
-class Plugin : public albert::util::ExtensionPlugin,
+class Plugin : public albert::ExtensionPlugin,
                public albert::GlobalQueryHandler
 {
     ALBERT_PLUGIN
 
 public:
 
-    Plugin();
+    void initialize() override;
 
     QString defaultTrigger() const override;
     QString synopsis(const QString &) const override;
-    void handleTriggerQuery(albert::Query &) override;
-    std::vector<albert::RankItem> handleGlobalQuery(const albert::Query &) override;
+    std::vector<albert::RankItem> rankItems(albert::QueryContext &) override;
     QWidget* buildConfigWidget() override;
 
 private:
 
     std::variant<QStringList, MathStructure>
-    runQalculateLocked(const albert::Query &, const EvaluationOptions &eo) ;
+    runQalculateLocked(const albert::QueryContext &, const EvaluationOptions &eo) ;
 
     std::shared_ptr<albert::Item> buildItem(const QString &query, MathStructure &mstruct) const;
 
@@ -35,5 +34,4 @@ private:
     EvaluationOptions eo;
     PrintOptions po;
     std::mutex qalculate_mutex;
-
 };
